import { Module, createRequire } from 'module';
import path from 'path';
import vm from 'vm';
import { nanoid } from 'nanoid';
import { transformSync, transform } from 'esbuild';

const requireFromString = (code, options = {}) => {
    const { globals = {} } = options;
    const mainModule = require.main;
    const fileName = path.join(mainModule.path, `${nanoid()}.js`);
    const contextModule = new Module(fileName, mainModule);
    contextModule.filename = fileName;
    contextModule.path = mainModule.path;
    contextModule.paths = mainModule.paths;
    contextModule.require = createRequire(fileName);
    vm.runInNewContext(code, Object.assign({ __dirname: contextModule.path, __filename: contextModule.filename, exports: contextModule.exports, module: contextModule, require: contextModule.require }, globals), { microtaskMode: 'afterEvaluate' });
    contextModule.loaded = true;
    return contextModule.exports;
};

/*! *****************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */

function __awaiter(thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
}

const importFromString = (code, options = {}) => __awaiter(void 0, void 0, void 0, function* () {
    const { globals, transformOptions = {} } = options;
    const transformResult = yield transform(code, Object.assign({ format: 'cjs' }, transformOptions));
    return requireFromString(transformResult.code, { globals });
});
const importFromStringSync = (code, options = {}) => {
    const { globals, transformOptions = {} } = options;
    const transformResult = transformSync(code, Object.assign({ format: 'cjs' }, transformOptions));
    return requireFromString(transformResult.code, { globals });
};

export { importFromString, importFromStringSync, requireFromString };
